import os
import sys
import errno

def is_pathname_valid(path):
    try:
        if not isinstance(path, str) or not path:
            return False
        
        _, path = os.path.splitdrive(path)

        root_dirname  = os.environ.get('HOMEDRIVE', 'C:') if sys.platform == 'win32' else os.path.sep

        if not os.path.isdir(root_dirname):
            raise ValueError('No idea of what happened.')
        
        root_dirname = root_dirname.rstrip(os.path.sep) + os.path.sep

        for path_part in path.split(os.path.sep):
            try:
                os.lstat(root_dirname + path_part)
            except OSError as exc:
                if hasattr(exc, 'winerror'):
                    if exc.winerror == 123:
                        return False
                elif exc.errno in {errno.ENAMETOOLONG, errno.ERANGE}:
                    return False
    except TypeError as exc:
        return False
    
    else:
        return True

def is_path_creatable(path):
    dirname = os.path.dirname(path) or os.getcwd()
    return os.access(dirname, os.W_OK)

def is_path_exists_or_creatable(path):
    try:
        return is_pathname_valid(path) and (os.path.exists(path) or is_path_creatable(path))
    except OSError:
        return False

def disable_print():
    sys.stdout = open(os.devnull, 'w')

def enable_print():
    sys.stdout = sys.__stdout__


from .errors import InputError
from .functions import str_at_least_n

def str_to_chapters(chapters, chapters_name, chapters_asked):
    
    pre_chapter_asked = []
    chapters_to_download = []
    if chapters_asked in ['', '-1', '*']:
        pre_chapter_asked = [(chapters[i], chapters_name[i]) for i in range(len(chapters))]
    else:
        try:
            chapters_asked = str_at_least_n(str(float(chapters_asked)), len(chapters_name[0]))
            i = chapters_name.index(chapters_asked)
            pre_chapter_asked = [(chapters[i], chapters_name[i])]
        except:
            for list in chapters_asked.split('/'):
                try:
                    begin, end = list.split('-')
                except:
                    raise InputError("The given format of chapters to download is not correct. Please check https://github.com/Boubou0909/Mangas-dl#answers-format.")
                
                try:
                    indice_begining = chapters_name.index(str_at_least_n(str(float(begin)), len(chapters_name[0])))
                    indice_ending = chapters_name.index(str_at_least_n(str(float(end)), len(chapters_name[0])))
                except:
                    raise InputError("At least one of the chapter(s) asked doesn't exist.")

                if len(str(end).split('.')) > 1:
                    k = 0.1
                    while k < 1:
                        try:
                            indice_ending = chapters.index(str(float(end) + round(k,1)))
                        except:
                            pass
                        
                        k += 0.1

                for i in range(indice_begining, indice_ending + 1):
                    pre_chapter_asked.append((chapters[i], chapters_name[i]))
        
    for i in range(len(pre_chapter_asked)):
        chapters_to_download.append((pre_chapter_asked[i][0], pre_chapter_asked[i][1]))

    return chapters_to_download


from setuptools import setup, find_packages

__version__ = "1.0.1"

with open("readme.md", "r") as f:
    readme = f.read()

requirements = ["requests-HTML>=0.10.0", "MangaDex.py>=2.0.6", "pillow>=8.0.1", "imgurpython>=1.1.7"]

setup(
    name = "mangas-dl",
    version = __version__,
    author = "Boubou0909",
    author_email = "balthazar0909@gmail.com",
    description = "Mangas' scans downloader app",
    long_description = readme,
    long_description_content_type = "text/markdown",
    url = "https://github.com/Boubou0909/Mangas-dl",
    packages = find_packages(),
    package_data = {"": ["mangas_dl/HELP.md", "mangas_dl/conf.ini", "mangas_dl/language_codes.json", "mangas_dl/settings.json", "mangas_dl/websites_used.json"]},
    include_package_data = True,
    install_requires = requirements,
    entry_points = '''
        [console_scripts]
        mangas-dl=mangas_dl.__main__:main
    ''',
    classifiers = 
    [
        "Programming Language :: Python :: 3.9"
    ]
)


import json
import os

from .pre_download_functions import * 
from .download_chapters_functions import * 

from .Headers.functions_web import does_page_exists
from .Headers.errors import UnknownWebsiteError, OSError, ConnexionError
from .Headers.functions import ask_until_y_or_n
from .Headers.functions_os import is_path_exists_or_creatable
from .Headers.functions_formatting import str_to_chapters

try:
    with open("mangas_dl/websites_used.json") as file:
        LAUNCH_FUNCTIONS = json.load(file)
        KNOWN_WEBSITES = LAUNCH_FUNCTIONS.keys()
except:
    raise FileNotFoundError("The file websites_used.json has not been found. Please make sure it exists before lauching mangas-dl.")

class Mangas_dl:
    def __init__(self, url):
        self.url = url

        pre_process = url.split('/')
        self.site_name = pre_process[2]

        self.manga_name = ""
        self.chapters = []
        self.chapters_name = []
        self.chapters_asked = []

        self.download_path = ""

    def test_connexion(self):
        return does_page_exists(self.url)

    def pre_download(self, choosen_language = -1):
        if not self.site_name in KNOWN_WEBSITES:
            raise UnknownWebsiteError(self.site_name)
        
        if not does_page_exists(self.url):
            raise ConnexionError(self.url)

        self.chapters, self.chapters_name, self.manga_name = eval(LAUNCH_FUNCTIONS[self.site_name]["pre_download"] + "(self.url, choosen_language)")

    def ask_chapters_to_download(self):
        if not hasattr(self, "chapters") or not hasattr(self, "chapters_name"):
            raise Exception("No idea how you arrived here.")

        print(len(self.chapters), "chapters have been found (from " + self.chapters_name[0] + " to " + self.chapters_name[-1] + ").")
        chapters_asked = input('Which chapter(s) would you like to download ? ')

        if chapters_asked in ("list", "List", "LIST"):
            try:
                for i in range(len(self.chapters) // 10 + 1):
                    for j in range(10):
                        print(self.chapters_name[10 * i + j], end="  ")
                    print("")
            except:
                print("  ".join(self.chapters_name[10 * (len(self.chapters_name) // 10):]))
            finally:
                chapters_asked = input('Which chapter(s) would you like to download ? ')
        self.chapters_asked = str_to_chapters(self.chapters, self.chapters_name, chapters_asked)

    def ask_path(self):
        download_path = ''
        use_json = False

        is_path_valid = False

        while not is_path_valid:
            try:
                f = open('mangas_dl/settings.json', 'r')
                settings = json.load(f)
                f.close()
                remembered_path = settings['remembered_path']
                use_remembered_path = ask_until_y_or_n('The path "' + remembered_path + '" is known. Do you want to use it ?')
                download_path = remembered_path if use_remembered_path else ''
                use_json = download_path != ''
            except:
                pass

            while download_path == '':
                download_path = input('Destination folder : ')
            
            if not is_path_exists_or_creatable(download_path):
                raise OSError("The given path doesn't exists and is not creatable.")

            if not os.path.exists(download_path):
                create_path = ask_until_y_or_n('The folder doesn\'t exist. Create it ?')
                if create_path:
                    os.makedirs(download_path)
                    is_path_valid = True
            else:
                is_path_valid = True
        
            if not use_json and is_path_valid:
                save_path = ask_until_y_or_n('Do you want to save this path ?')
                if save_path:
                    f = open('settings.json', 'a+')
                    saved_path = {'remembered_path' : download_path}
                    json.dump(saved_path, f)
                    f.close()

        self.download_path = download_path.rstrip(os.path.sep) + os.path.sep

        manga_name = input("Name of the final folder : ")
        if manga_name != "":
            self.manga_name = manga_name

    def download_chapters(self):
        if not hasattr(self, "chapters_asked") or not hasattr(self, "download_path"):
            raise Exception("No idea how you arrived here")

        eval(LAUNCH_FUNCTIONS[self.site_name]["download_chapters"] + "(self.url, self.chapters_asked, self.download_path, self.manga_name)")

        return "Download finished successfully. Enjoy !"


import os
import sys, getopt
import json

from .mangas_dl_class import Mangas_dl

from .Headers.errors import ConnexionError
from .Headers.functions_formatting import str_to_chapters
from .Headers.functions_os import is_path_exists_or_creatable

__version__ = "1.0.1"

ARGS = sys.argv[1:]

sys.path.append(os.path.dirname(os.path.realpath(__file__)))

def main_one_line(argv):
    try:
        opts, args = getopt.getopt(argv, "hvl:c:p:n:", ["help", "version", "language", "chapters", "path", "list", "name"])
    except getopt.GetoptError:
        print("\nUsage:\n\tmangas-dl [options] URL")
        print("or:\n\tmangas-dl <command> [options]")
        print("\nNo such option: \"" + argv[0] + "\"")
        sys.exit(1)

    if argv[0] in ("save_path", "save_language"):
        try:
            with open('mangas_dl/settings.json', 'r+') as f:
                settings = json.load(f)
                f.seek(0)

                if argv[0] == "save_path":
                    if not is_path_exists_or_creatable(argv[1]):
                        print("The given path doesn't exist and is not creatable. Please try again.")
                        sys.exit(1)
                    settings["remembered_path"] = argv[1]
                    json.dump(settings, f)
                    print("Path \"" + argv[1] + "\" learnt")
                elif argv[0] == "save_language":
                    try:
                        with open("mangas_dl/language_codes.json") as file:
                            LANGUAGE_CODES = json.load(file)
                    except:
                        raise FileNotFoundError("The file websites_used.json has not been found. Please make sure it exists before lauching mangas-dl.")
                    if not argv[1] in LANGUAGE_CODES.keys():
                        print("The given language is not taken in charge. Please read the complete list here : https://github.com/Boubou0909/Mangas-dl/blob/main/mangas_dl/language_codes.json")
                        sys.exit(1)
                    settings["remembered_language"] = argv[1]
                    json.dump(settings, f)
                    print("Language \"" + argv[1] + "\" learnt")

                f.truncate()
                f.close()
        except IOError:
            print("ERROR : Impossible to deal the request. Please try again later.")
            sys.exit(1)
        except IndexError:
            print("ERROR : Please give the what you want to save after the command.")
            print("\nUsage:\n\tmangas_dl <command> [options]")
            sys.exit(1)
        except:
            print("ERROR : try \"pip install mangas-dl --upgrade\" to update the package")
            sys.exit(1)
        finally:
            sys.exit(0)

    try:
        with open("mangas_dl/settings.json", "r") as f:
            settings = json.load(f)
            language = settings["remembered_language"]
    except:
        language = "en"
    
    chapters_asked = ""
    listing = False
    manga_name = ""
    for opt, arg in opts:
        if opt in ("-h", "--help"):
            f = open("mangas_dl/HELP.md", "r")
            print(f.read())
            f.close()
            sys.exit(0)
        elif opt in ("-v", "--version"):
            print("mangas-dl", __version__)
            sys.exit(0)
        elif opt in ("-l", "--language"):
            language = arg
        elif opt in ("-c", "--chapters"):
            chapters_asked = arg
        elif opt in ("-p", "--path"):
            if arg == "%":
                try:
                    f = open("mangas_dl/settings.json", "r")
                    settings = json.load(f)
                    f.close()
                    download_path = settings["remembered_path"]
                except:
                    raise OSError("There is no remebered path. Save one before using this command.")
            elif is_path_exists_or_creatable(arg):
                download_path = arg
            else:
                print("The given path does not exist and is not creatable. Please try again.")
                sys.exit(1)
        elif opt in ("--list"):
            listing = True
        elif opt in ("-n", "--name"):
            manga_name = arg
    
    if len(args) != 1:
        print("Please enter only one url at time.")
        sys.exit(1)
    elif not "download_path" in locals() and not listing:
        print("Please enter a path where scans will be saved.")
        sys.exit(1)

    url = args[0]

    mangas_dl = Mangas_dl(url)
    if not mangas_dl.test_connexion():
        raise ConnexionError(url)

    mangas_dl.pre_download(choosen_language = language)
    if len(mangas_dl.chapters) == 0:
        print("The remembered language is not available for this scan. English will be used instead.")
        mangas_dl.pre_download(choosen_language = "en")

    if listing:
        try:
            for i in range(len(mangas_dl.chapters) // 10 + 1):
                for j in range(10):
                    print(mangas_dl.chapters_name[10 * i + j], end="  ")
                print("")
        except:
            pass
        finally:
            sys.exit(0)

    if manga_name != "":
        mangas_dl.manga_name = manga_name

    mangas_dl.chapters_asked = str_to_chapters(mangas_dl.chapters, mangas_dl.chapters_name, chapters_asked)
    mangas_dl.download_path = download_path.rstrip(os.path.sep) + os.path.sep

    return mangas_dl.download_chapters()

def main_interactive(url):
    mangas_dl = Mangas_dl(url)
    if not mangas_dl.test_connexion():
        raise ConnexionError(url)

    mangas_dl.pre_download()
    mangas_dl.ask_chapters_to_download()
    mangas_dl.ask_path()

    return mangas_dl.download_chapters()

def main():
    if len(ARGS) == 0:
        print(main_interactive(input("Enter the main page of the manga you want to download : ")))
    else:
        print(main_one_line(ARGS))

if __name__ == "__main__":
    main()